#include <stdio.h>
#include <errno.h>
#include <string.h>
#include <ctype.h>

#define	KILOBYTE			1024
#define LINELEN				256
#define	SPLITBYTES_INIT		(633 * KILOBYTE)

/* about filesystem */
#ifdef __MSDOS__
# define DOSEXT_INIT		1
# define DIRDELIM			'\\'
#else
# ifdef unix
#  define DOSEXT_INIT		0
#  define DIRDELIM			'/'
# endif
#endif

#define	OPTION_ERR			1
#define	WRITE_FAIL			2
#define	INOPEN_ERR			4
#define	OUTOPEN_ERR			8

#define	eputs(s)		(void)fputs((s), stderr)
#define	streq(s,t)		(strcmp((s), (t)) == 0)

extern	long	strtol();
extern	void	exit();
extern	int		access();

static unsigned long	SplitBytes = SPLITBYTES_INIT;
static char				FName[LINELEN];
static int				Verbose = 0;
static int				DosExt	= DOSEXT_INIT;

/*
 * this function has many MAGIC NUMBERS, but I believe
 * it is more simple than #defineing.
 */
	static int
OptChar(s)
	char	*s;
{
	if (*s == 'h') {
		eputs(" slit - binary file split. (C) TOYODA Eizi 1994\n");
		eputs(" usage:\n");
		eputs("  slit [-evVmMdD] [-splitsize] [files...]\n");
		eputs("  splitsize is in kilobytes.\n");
		eputs(" options:\n");
#ifndef __MSDOS__
		eputs("  -e MS-DOS like filename\n");
#endif
		eputs("  -v verbose output\n");
		eputs("  -V splitsize 1440*1024 (for IBM & DOS/V?)\n");
		eputs("  -m splitsize 1220*1024 (for PC98 2HD)\n");
		eputs("  -M splitsize 1184*1024 (for IBM 2HD(2HC))\n");
		eputs("  -d splitsize 712*1024 (for Mac &c. 2DD)\n");
		eputs("  -D splitsize 633*1024 (for PC98 2DD -- default)\n");
		exit(0);
	} else if (*s == 'e') {
		DosExt = 1;
	} else if (*s == 'v') {
		Verbose = 1;
	/*
	 * magic numbers of split size
	 */
	} else if (*s == 'V') {
		SplitBytes = 1440 * KILOBYTE;
	} else if (*s == 'm') {
		SplitBytes = 1220 * KILOBYTE;
	} else if (*s == 'M') {
		SplitBytes = 1184 * KILOBYTE;
	} else if (*s == 'd') {
		SplitBytes = 712 * KILOBYTE;
	} else if (*s == 'D') {
		SplitBytes = 633 * KILOBYTE;
	} else if (*s == 'b') {
	} else if (isxdigit(*s)) {
		SplitBytes = KILOBYTE * strtol(s, (char **)NULL, 0);
		if (SplitBytes == 0) {
			eputs("split by 0 is meaningless\n");
			exit(OPTION_ERR);
		}
		return -1;	/* nokori wo suteru */
	} else {
		eputs(s);
		eputs(" : undefined option(ignored). slit -h to help.\n");
		return OPTION_ERR;
	}
	return 0;
}

	static int
Option(opts)
	char	*opts;
{
	opts++;
	while (*opts) {
		int		r;
		if ((r = OptChar(opts)) > 0)
			return OPTION_ERR;
		if (r < 0)
			break;
		opts++;
	}
	return 0;
}

	static FILE	*
OutFile(outname, files)
	char	*outname;
	int		files;
{
	char	*baseptr, *extptr;
	if (files != 0)
		outname = FName;
	if ((baseptr = strrchr(outname, DIRDELIM)) == NULL)
		baseptr = outname;
	else
		baseptr++;
	if (DosExt) {
		if ((extptr = strchr(baseptr, '.')) != NULL)
			*extptr = '\0';
		else
			extptr = baseptr + strlen(baseptr);
	} else {
		if ((extptr = strrchr(baseptr, '.')) == NULL)
			extptr = baseptr + strlen(baseptr);
	};
	if (files == 0) {
		(void)sprintf(FName, "%s.%03i", baseptr, files);
	} else {
		(void)sprintf(extptr, ".%03i", files);
	}
	if (Verbose) {
		(void)fprintf(stderr, "%s %s",
			((access(FName, 0) == 0) ? "overwrite" : "open"),
			FName);
		(void)fputc('\n', stderr);
	}
	return fopen(FName, "wb");
}

	int
FileLoop(infilename, input)
	char	*infilename;
	FILE	*input;
{
	long	bytes = 0;
	FILE	*output;
	int		c, files = 0;
	output = OutFile(infilename, files);
	while ((c = getc(input)) != EOF) {
		if (bytes == SplitBytes) {
			(void)fclose(output);
			output = OutFile(infilename, ++files);
			if (output == NULL) {
				perror("output fopen");
				return WRITE_FAIL;
			}
			bytes = 0;
		}
		if (Examine(fputc(c, output), "fputc")) {
			(void)Examine(fclose(output), "fclose");
			(void)fclose(input);
			return WRITE_FAIL;
		};
		bytes++;
	}
	if (Examine(fclose(output), "fclose"))
		return WRITE_FAIL;
	(void)fclose(input);
	return 0;
}

	static int
Examine(eofiserr, note)
	int		eofiserr;
	char	*note;
{
	if (eofiserr != EOF)
		return 0;
	perror(note);
	return WRITE_FAIL;
}

	int
main(argc, argv)
	int		argc;
	char	*argv[];
{
	int		i, retcode = 0, infiles = 0;
	FILE	*input;
	for (i=1; i<argc; i++) {
		if (argv[i][0] == '-') {
			retcode |= Option(argv[i]);
		} else {
			if ((input = fopen(argv[i], "rb")) == NULL) {
				perror("input fopen");
				retcode |= INOPEN_ERR;
			} else {
				retcode |= FileLoop(argv[i], input);
				infiles++;
			};
		}
	}
	if (infiles == 0) {
		retcode = FileLoop("stdin", stdin);
	}
	return retcode;
}
